@@ -30,9 +30,9 @@
 #include "gdbuserror.h"
 #include "gdbusprivate.h"
 #include "gdbusconnection.h"
+#include "gio-marshal.h"
 
 #include "glibintl.h"
-#include "gioalias.h"
 
 /**
  * SECTION:gdbusnamewatching
@@ -448,6 +448,7 @@ has_connection (Client *client)
                                                                                    "NameOwnerChanged",      /* signal */
                                                                                    "/org/freedesktop/DBus", /* path */
                                                                                    client->name,
+                                                                                   G_DBUS_SIGNAL_FLAGS_NONE,
                                                                                    on_name_owner_changed,
                                                                                    client,
                                                                                    NULL);
@@ -590,7 +591,7 @@ g_bus_watch_name (GBusType                  bus_type,
 
 /**
  * g_bus_watch_name_on_connection:
- * @connection: A #GDBusConnection that is not closed.
+ * @connection: A #GDBusConnection.
  * @name: The name (well-known or unique) to watch.
  * @flags: Flags from the #GBusNameWatcherFlags enumeration.
  * @name_appeared_handler: Handler to invoke when @name is known to exist or %NULL.
@@ -635,9 +636,8 @@ guint g_bus_watch_name_on_connection (GDBusConnection          *connection,
     g_main_context_ref (client->main_context);
 
   if (map_id_to_client == NULL)
-    {
-      map_id_to_client = g_hash_table_new (g_direct_hash, g_direct_equal);
-    }
+    map_id_to_client = g_hash_table_new (g_direct_hash, g_direct_equal);
+
   g_hash_table_insert (map_id_to_client,
                        GUINT_TO_POINTER (client->id),
                        client);
@@ -655,6 +655,33 @@ typedef struct {
   GClosure *name_vanished_closure;
 } WatchNameData;
 
+static WatchNameData *
+watch_name_data_new (GClosure *name_appeared_closure,
+                     GClosure *name_vanished_closure)
+{
+  WatchNameData *data;
+
+  data = g_new0 (WatchNameData, 1);
+
+  if (name_appeared_closure != NULL)
+    {
+      data->name_appeared_closure = g_closure_ref (name_appeared_closure);
+      g_closure_sink (name_appeared_closure);
+      if (G_CLOSURE_NEEDS_MARSHAL (name_appeared_closure))
+        g_closure_set_marshal (name_appeared_closure, _gio_marshal_VOID__STRING_STRING);
+    }
+
+  if (name_vanished_closure != NULL)
+    {
+      data->name_vanished_closure = g_closure_ref (name_vanished_closure);
+      g_closure_sink (name_vanished_closure);
+      if (G_CLOSURE_NEEDS_MARSHAL (name_vanished_closure))
+        g_closure_set_marshal (name_vanished_closure, _gio_marshal_VOID__STRING);
+    }
+
+  return data;
+}
+
 static void
 watch_with_closures_on_name_appeared (GDBusConnection *connection,
                                       const gchar     *name,
@@ -674,6 +701,10 @@ watch_with_closures_on_name_appeared (GDBusConnection *connection,
   g_value_set_string (&params[2], name_owner);
 
   g_closure_invoke (data->name_appeared_closure, NULL, 3, params, NULL);
+
+  g_value_unset (params + 0);
+  g_value_unset (params + 1);
+  g_value_unset (params + 2);
 }
 
 static void
@@ -691,6 +722,9 @@ watch_with_closures_on_name_vanished (GDBusConnection *connection,
   g_value_set_string (&params[1], name);
 
   g_closure_invoke (data->name_vanished_closure, NULL, 2, params, NULL);
+
+  g_value_unset (params + 0);
+  g_value_unset (params + 1);
 }
 
 static void
@@ -734,28 +768,18 @@ g_bus_watch_name_with_closures (GBusType                 bus_type,
                                 GClosure                *name_appeared_closure,
                                 GClosure                *name_vanished_closure)
 {
-  WatchNameData *data;
-
-  data = g_new0 (WatchNameData, 1);
-
-  if (name_appeared_closure != NULL)
-    data->name_appeared_closure = g_closure_ref (name_appeared_closure);
-
-  if (name_vanished_closure != NULL)
-    data->name_vanished_closure = g_closure_ref (name_vanished_closure);
-
   return g_bus_watch_name (bus_type,
           name,
           flags,
           name_appeared_closure != NULL ? watch_with_closures_on_name_appeared : NULL,
           name_vanished_closure != NULL ? watch_with_closures_on_name_vanished : NULL,
-          data,
+          watch_name_data_new (name_appeared_closure, name_vanished_closure),
           bus_watch_name_free_func);
 }
 
 /**
  * g_bus_watch_name_on_connection_with_closures:
- * @connection: A #GDBusConnection that is not closed.
+ * @connection: A #GDBusConnection.
  * @name: The name (well-known or unique) to watch.
  * @flags: Flags from the #GBusNameWatcherFlags enumeration.
  * @name_appeared_closure: (allow-none): #GClosure to invoke when @name is known
@@ -780,22 +804,12 @@ guint g_bus_watch_name_on_connection_with_closures (
                                       GClosure                 *name_appeared_closure,
                                       GClosure                 *name_vanished_closure)
 {
-  WatchNameData *data;
-
-  data = g_new0 (WatchNameData, 1);
-
-  if (name_appeared_closure != NULL)
-    data->name_appeared_closure = g_closure_ref (name_appeared_closure);
-
-  if (name_vanished_closure != NULL)
-    data->name_vanished_closure = g_closure_ref (name_vanished_closure);
-
   return g_bus_watch_name_on_connection (connection,
           name,
           flags,
           name_appeared_closure != NULL ? watch_with_closures_on_name_appeared : NULL,
           name_vanished_closure != NULL ? watch_with_closures_on_name_vanished : NULL,
-          data,
+          watch_name_data_new (name_appeared_closure, name_vanished_closure),
           bus_watch_name_free_func);
 }
 
@@ -837,6 +851,3 @@ g_bus_unwatch_name (guint watcher_id)
       client_unref (client);
     }
 }
-
-#define __G_DBUS_NAME_WATCHING_C__
-#include "gioaliasdef.c"
