@@ -31,7 +31,7 @@
 #include <gio/gasyncresult.h>
 
 
-G_DEFINE_TYPE (GIOStream, g_io_stream, G_TYPE_OBJECT);
+G_DEFINE_ABSTRACT_TYPE (GIOStream, g_io_stream, G_TYPE_OBJECT);
 
 /**
  * SECTION:giostream
@@ -222,7 +222,8 @@ g_io_stream_is_closed (GIOStream *stream)
  * Gets the input stream for this object. This is used
  * for reading.
  *
- * Returns: a #GInputStream, owned by the #GIOStream. Do not free.
+ * Returns: (transfer none): a #GInputStream, owned by the #GIOStream.
+ * Do not free.
  *
  * Since: 2.22
  */
@@ -245,7 +246,8 @@ g_io_stream_get_input_stream (GIOStream *stream)
  * Gets the output stream for this object. This is used for
  * writing.
  *
- * Returns: a #GOutputStream, owned by the #GIOStream. Do not free.
+ * Returns: (transfer none): a #GOutputStream, owned by the #GIOStream.
+ * Do not free.
  *
  * Since: 2.22
  */
@@ -495,11 +497,10 @@ g_io_stream_close_async (GIOStream           *stream,
 
   if (!g_io_stream_set_pending (stream, &error))
     {
-      g_simple_async_report_gerror_in_idle (G_OBJECT (stream),
+      g_simple_async_report_take_gerror_in_idle (G_OBJECT (stream),
 					    callback,
 					    user_data,
 					    error);
-      g_error_free (error);
       return;
     }
 
@@ -568,10 +569,7 @@ close_async_thread (GSimpleAsyncResult *res,
     {
       result = class->close_fn (G_IO_STREAM (object), cancellable, &error);
       if (!result)
-	{
-	  g_simple_async_result_set_from_error (res, error);
-	  g_error_free (error);
-	}
+        g_simple_async_result_take_error (res, error);
     }
 }
 
