@@ -106,10 +106,8 @@ g_socket_address_enumerator_real_next_async (GSocketAddressEnumerator *enumerato
   if (address)
     g_simple_async_result_set_op_res_gpointer (result, address, NULL);
   else if (error)
-    {
-      g_simple_async_result_set_from_error (result, error);
-      g_error_free (error);
-    }
+    g_simple_async_result_take_error (result, error);
+
   g_simple_async_result_complete_in_idle (result);
   g_object_unref (result);
 }
