@@ -38,7 +38,6 @@
 #include "gasynchelper.h"
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gunixoutputstream
@@ -436,7 +435,7 @@ write_async_cb (WriteAsyncData *data,
 	  
 	  g_set_error (&error, G_IO_ERROR,
 		       g_io_error_from_errno (errsv),
-		       _("Error reading from unix: %s"),
+		       _("Error writing to unix: %s"),
 		       g_strerror (errsv));
 	}
       break;
@@ -450,10 +449,7 @@ write_async_cb (WriteAsyncData *data,
   g_simple_async_result_set_op_res_gssize (simple, count_written);
 
   if (count_written == -1)
-    {
-      g_simple_async_result_set_from_error (simple, error);
-      g_error_free (error);
-    }
+    g_simple_async_result_take_error (simple, error);
 
   /* Complete immediately, not in idle, since we're already in a mainloop callout */
   g_simple_async_result_complete (simple);
@@ -488,6 +484,7 @@ g_unix_output_stream_write_async (GOutputStream       *stream,
   source = _g_fd_source_new (unix_stream->priv->fd,
 			     G_IO_OUT,
 			     cancellable);
+  g_source_set_name (source, "GUnixOutputStream");
   
   g_source_set_callback (source, (GSourceFunc)write_async_cb, data, g_free);
   g_source_attach (source, g_main_context_get_thread_default ());
@@ -557,10 +554,7 @@ close_async_cb (CloseAsyncData *data)
 				      g_unix_output_stream_close_async);
 
   if (!result)
-    {
-      g_simple_async_result_set_from_error (simple, error);
-      g_error_free (error);
-    }
+    g_simple_async_result_take_error (simple, error);
 
   /* Complete immediately, not in idle, since we're already in a mainloop callout */
   g_simple_async_result_complete (simple);
@@ -599,6 +593,3 @@ g_unix_output_stream_close_finish (GOutputStream  *stream,
   /* Failures handled in generic close_finish code */
   return TRUE;
 }
-
-#define __G_UNIX_OUTPUT_STREAM_C__
-#include "gioaliasdef.c"
